from functools import cached_property
from typing import Optional

from pydantic import ValidationError

from .._config import Config
from .._execution_context import ExecutionContext
from .._services import (
    ApiClient,
    AssetsService,
    AttachmentsService,
    BucketsService,
    ConnectionsService,
    ContextGroundingService,
    ConversationsService,
    DocumentsService,
    EntitiesService,
    FolderService,
    JobsService,
    McpService,
    ProcessesService,
    QueuesService,
    ResourceCatalogService,
    TasksService,
    UiPathLlmChatService,
    UiPathOpenAIService,
)
from .._utils._auth import resolve_config
from .errors import BaseUrlMissingError, SecretMissingError


class UiPath:
    def __init__(
        self,
        *,
        base_url: Optional[str] = None,
        secret: Optional[str] = None,
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
        scope: Optional[str] = None,
        debug: bool = False,
    ) -> None:
        try:
            base_url, secret = resolve_config(
                base_url, secret, client_id, client_secret, scope
            )
            self._config = Config(
                base_url=base_url,
                secret=secret,
            )
        except ValidationError as e:
            for error in e.errors():
                if error["loc"][0] == "base_url":
                    raise BaseUrlMissingError() from e
                elif error["loc"][0] == "secret":
                    raise SecretMissingError() from e
        self._execution_context = ExecutionContext()

    @property
    def api_client(self) -> ApiClient:
        return ApiClient(self._config, self._execution_context)

    @property
    def assets(self) -> AssetsService:
        return AssetsService(self._config, self._execution_context)

    @cached_property
    def attachments(self) -> AttachmentsService:
        return AttachmentsService(self._config, self._execution_context)

    @property
    def processes(self) -> ProcessesService:
        return ProcessesService(self._config, self._execution_context, self.attachments)

    @property
    def tasks(self) -> TasksService:
        return TasksService(self._config, self._execution_context)

    @cached_property
    def buckets(self) -> BucketsService:
        return BucketsService(self._config, self._execution_context)

    @cached_property
    def connections(self) -> ConnectionsService:
        return ConnectionsService(self._config, self._execution_context, self.folders)

    @property
    def context_grounding(self) -> ContextGroundingService:
        return ContextGroundingService(
            self._config,
            self._execution_context,
            self.folders,
            self.buckets,
        )

    @property
    def documents(self) -> DocumentsService:
        return DocumentsService(self._config, self._execution_context)

    @property
    def queues(self) -> QueuesService:
        return QueuesService(self._config, self._execution_context)

    @property
    def jobs(self) -> JobsService:
        return JobsService(self._config, self._execution_context)

    @cached_property
    def folders(self) -> FolderService:
        return FolderService(self._config, self._execution_context)

    @property
    def llm_openai(self) -> UiPathOpenAIService:
        return UiPathOpenAIService(self._config, self._execution_context)

    @property
    def llm(self) -> UiPathLlmChatService:
        return UiPathLlmChatService(self._config, self._execution_context)

    @property
    def entities(self) -> EntitiesService:
        return EntitiesService(self._config, self._execution_context)

    @cached_property
    def resource_catalog(self) -> ResourceCatalogService:
        return ResourceCatalogService(
            self._config, self._execution_context, self.folders
        )

    @property
    def conversational(self) -> ConversationsService:
        return ConversationsService(self._config, self._execution_context)

    @property
    def mcp(self) -> McpService:
        return McpService(self._config, self._execution_context, self.folders)



"""UiPath Orchestrator Models.

This module contains models related to UiPath Orchestrator services.
"""

from .assets import Asset, UserAsset
from .attachment import Attachment
from .buckets import Bucket, BucketFile
from .job import Job, JobErrorInfo
from .mcp import McpServer, McpServerStatus, McpServerType
from .processes import Process
from .queues import (
    CommitType,
    QueueItem,
    QueueItemPriority,
    TransactionItem,
    TransactionItemResult,
)

__all__ = [
    "Asset",
    "UserAsset",
    "Attachment",
    "Bucket",
    "BucketFile",
    "Job",
    "JobErrorInfo",
    "Process",
    "CommitType",
    "QueueItem",
    "QueueItemPriority",
    "TransactionItem",
    "TransactionItemResult",
    "McpServer",
    "McpServerStatus",
    "McpServerType",
]



"""UiPath LLM Gateway Models.

This module contains models related to UiPath LLM Gateway service.
"""

from .llm_gateway import (
    AutoToolChoice,
    ChatCompletion,
    ChatCompletionChoice,
    ChatCompletionUsage,
    ChatMessage,
    EmbeddingItem,
    EmbeddingUsage,
    RequiredToolChoice,
    SpecificToolChoice,
    TextEmbedding,
    ToolCall,
    ToolChoice,
    ToolDefinition,
    ToolFunctionDefinition,
    ToolParametersDefinition,
    ToolPropertyDefinition,
)

__all__ = [
    "ToolPropertyDefinition",
    "ToolParametersDefinition",
    "ToolFunctionDefinition",
    "ToolDefinition",
    "AutoToolChoice",
    "RequiredToolChoice",
    "SpecificToolChoice",
    "ChatMessage",
    "ChatCompletionChoice",
    "ChatCompletionUsage",
    "ChatCompletion",
    "EmbeddingItem",
    "EmbeddingUsage",
    "TextEmbedding",
    "ToolChoice",
    "ToolCall",
]



"""UiPath Common Models.

This module contains common models used across multiple services.
"""

from .auth import TokenData
from .interrupt_models import (
    CreateEscalation,
    CreateTask,
    InvokeProcess,
    WaitEscalation,
    WaitJob,
    WaitTask,
)
from .paging import PagedResult

__all__ = [
    "TokenData",
    "CreateTask",
    "CreateEscalation",
    "WaitEscalation",
    "InvokeProcess",
    "WaitTask",
    "WaitJob",
    "PagedResult",
]
